package sun.awt.datatransfer;

import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.awt.datatransfer.DataFlavor;
import java.util.HashMap;
import java.awt.datatransfer.Transferable;

public class ClipboardTransferable implements Transferable
{
    private final HashMap flavorsToData;
    private DataFlavor[] flavors;
    
    public ClipboardTransferable(SunClipboard sunClipboard) {
        this.flavorsToData = new HashMap();
        this.flavors = new DataFlavor[0];
        sunClipboard.openClipboard(null);
        try {
            long[] clipboardFormats = sunClipboard.getClipboardFormats();
            if (clipboardFormats != null && clipboardFormats.length > 0) {
                HashMap<Object, Object> hashMap = new HashMap(clipboardFormats.length, 1.0f);
                Map<Object, Object> flavorsForFormats = DataTransferer.getInstance().getFlavorsForFormats(clipboardFormats, SunClipboard.getDefaultFlavorTable());
                for (DataFlavor dataFlavor : flavorsForFormats.keySet()) {
                    this.fetchOneFlavor(sunClipboard, dataFlavor, (Long)flavorsForFormats.get(dataFlavor), hashMap);
                }
                DataTransferer.getInstance();
                this.flavors = DataTransferer.setToSortedDataFlavorArray(this.flavorsToData.keySet());
            }
        }
        finally {
            sunClipboard.closeClipboard();
        }
    }
    
    private boolean fetchOneFlavor(SunClipboard sunClipboard, DataFlavor key, Long key2, HashMap hashMap) {
        if (!this.flavorsToData.containsKey(key)) {
            long longValue = (long)key2;
            Object o = null;
            if (!hashMap.containsKey(key2)) {
                try {
                    o = sunClipboard.getClipboardData(longValue);
                }
                catch (IOException ex) {
                    o = ex;
                }
                catch (Throwable t) {
                    t.printStackTrace();
                }
                hashMap.put(key2, o);
            }
            else {
                o = hashMap.get(key2);
            }
            if ((o instanceof IOException)) {
                this.flavorsToData.put(key, o);
                return false;
            }
            if (o != null) {
                this.flavorsToData.put(key, new ClipboardTransferable.DataFactory(longValue, (byte[])(byte[])o));
                return true;
            }
        }
        return false;
    }
    
    @Override
    public DataFlavor[] getTransferDataFlavors() {
        return (DataFlavor[])(DataFlavor[])this.flavors.clone();
    }
    
    @Override
    public boolean isDataFlavorSupported(DataFlavor key) {
        return this.flavorsToData.containsKey(key);
    }
    
    @Override
    public Object getTransferData(DataFlavor dataFlavor) throws UnsupportedFlavorException, IOException {
        if (!this.isDataFlavorSupported(dataFlavor)) {
            throw new UnsupportedFlavorException(dataFlavor);
        }
        Object o = this.flavorsToData.get(dataFlavor);
        if ((o instanceof IOException)) {
            throw (IOException)o;
        }
        if ((o instanceof ClipboardTransferable.DataFactory)) {
            o = ((ClipboardTransferable.DataFactory)o).getTransferData(dataFlavor);
            return o;
        }
        return o;
    }
    
    private final class DataFactory
    {
        final long format;
        final byte[] data;
        
        DataFactory(long format, byte[] data) {
            this.format = format;
            this.data = data;
        }
        
        public Object getTransferData(DataFlavor dataFlavor) throws IOException {
            return DataTransferer.getInstance().translateBytes(this.data, dataFlavor, this.format, ClipboardTransferable.this);
        }
    }
}
